import numpy as np

def sigmoid(x):
    z = (1/(1 + np.exp(-x)))
    cache=x
    return z,cache

def binary_step(x):
    if x<0:
        return 0
    else:
        return 1

def tanh(x):
    z = (2/(1 + np.exp(-2*x))) -1
    cache=x
    return z,cache

def relu(x):
    z = np.maximum(0,x)
    cache=x
    return z,cache

def elu(x, a):
    if x<0:
        return a*(np.exp(x)-1)
    else:
        return x

def softmax(x):
    z = np.exp(x)
    z_ = z/z.sum()
    cache=x
    return z_,cache

def relu_backward(dA,cache):
    Z = cache
    dZ = np.array(dA, copy=True)
    dZ[Z <= 0] = 0

    return dZ

def sigmoid_backward(dA, cache):
    Z = cache
    
    s = 1/(1+np.exp(-Z))
    dZ = dA * s * (1-s)
        
    return dZ
def tanh_backward(dA,cache):
    Z=cache
    t=(np.exp(Z)-np.exp(-Z))/(np.exp(Z)+np.exp(-Z))
    dZ=1-t**2

    return dZ




from .activation_functions import *
import numpy as np

def compute_cost(AL,Y):
    m = Y.shape[1]

    cost = (1./m) * (-np.dot(Y,np.log(AL).T) - np.dot(1-Y, np.log(1-AL).T))
    cost = np.squeeze(cost)   
    return cost

def linear_backward(dZ, cache):
    A_prev, W, b = cache
    m = A_prev.shape[1]

    dW = (1/m)*np.dot(dZ,A_prev.T)
    db = (1/m)*np.sum(dZ,axis=1,keepdims=True)
    dA_prev = np.dot(W.T,dZ)
     
    return dA_prev, dW, db

def backward_activation(dA,cache,activation):
    linear_cache,activation_cache=cache

    dZ=eval(f"{activation}_backward(dA,activation_cache)")
    dA_prev, dW, db = linear_backward(dZ,linear_cache)

    return dA_prev, dW, db

def layer_seq(layers):
    sequence=[]
    for i in layers:
        b,a=i.split("*")[-1],i.split("*")[0]
        for y in range(int(b)):
            sequence.append(a)

    return sequence


def backward_model(AL,Y,caches,layer_func):
    grads = {}
    sequence=layer_seq(layer_func)
    L = len(caches)
    m = AL.shape[1]
    Y = Y.reshape(AL.shape)
    dAL = - (np.divide(Y, AL) - np.divide(1 - Y, 1 - AL)) 
    current_cache = caches[L-1]
    grads["dA" + str(L-1)], grads["dW" + str(L)], grads["db" + str(L)] = backward_activation(dAL, current_cache, activation = sequence[-1])
    for l in reversed(range(L-1)):
        current_cache = caches[l]
        dA_prev_temp, dW_temp, db_temp = backward_activation(grads["dA" + str(l + 1)], current_cache, activation = sequence[l])
        
        grads["dA" + str(l)] = dA_prev_temp
        grads["dW" + str(l + 1)] = dW_temp
        grads["db" + str(l + 1)] = db_temp
    return grads

def update_parameters(parameters, grads, learning_rate):
    
    L = len(parameters) // 2

    for l in range(L):
        parameters["W" + str(l+1)] = parameters["W" + str(l+1)] - learning_rate * grads["dW" + str(l + 1)]
        parameters["b" + str(l+1)] = parameters["b" + str(l+1)] - learning_rate * grads["db" + str(l + 1)]
    return parameters



import numpy as np
def hello(se=["relu*2","sigmoid*1"]):
    sequence=[]
    for i in se:
        b,a=i.split("*")[-1],i.split("*")[0]
        for y in range(int(b)):
            sequence.append(a)

    return sequence
print(np.log(1.00000000e+000))



from projecttalos.nn_funcs.forward_prop import param_init,forward_model
from projecttalos.nn_funcs.backward_prop import backward_model,update_parameters,compute_cost
from projecttalos.data_preprocessing import ImagePreprocess
import numpy as np



class NeuralNetwork:
    def __init__(self,layer_dims,layer_func,learning_rate,iteration,train_score=None,test_score=None):
        self.layer_func=layer_func
        self.layer_dims=layer_dims
        self.learning_rate=learning_rate
        self.iteration=iteration
        self.test_score=test_score
        self.train_score=train_score
    def train(self,X,Y):
        costs=[]
        parameters=param_init(self.layer_dims)
        for i in range(self.iteration+1):
            AL, caches = forward_model(X,parameters,self.layer_func)
            cost = compute_cost(AL,Y)
            grads = backward_model(AL,Y,caches,self.layer_func)

            parameters = update_parameters(parameters,grads,self.learning_rate)
            costs.append(cost)
            if i%100 == 0:
                print (f"Cost after iteration {i}: {cost}")
        self.train_score=np.divide((np.sum(costs)-np.mean(costs)),np.sum(costs))*100
        print(f"Training Accuracy: {self.train_score}")    
        return parameters
    def predict(self,X,parameters):
        result,cache=forward_model(X,parameters,self.layer_func)
        return result
    def score(self,X,Y,parameters):
        result,cache=forward_model(X,parameters,self.layer_func)
        self.test_score=np.divide(np.sum(Y)-np.mean(np.abs(np.subtract(Y,result))),np.sum(Y))*100
        print(f"Accuracy:{self.test_score}")
        



import numpy as np
import os
import cv2
import random 

def ImagePreprocess(path2img,train_size=0.7,size=(80,80)):
    x_test=[]
    y_test=[[]]
    x_train=[]
    y_train=[[]]
    for path in path2img:
        numberoffiles=len(os.listdir(path))
        counter=0
        for img in os.listdir(path):
            if counter<(numberoffiles*train_size):
                pic = cv2.cvtColor(cv2.imread(os.path.join(path,img)),cv2.COLOR_BGR2RGB)
                pic = cv2.imread(os.path.join(path,img))
                pic = cv2.resize(pic,size)
                x_train.append([pic])
                y_train[0].append(path2img.index(path))
                counter+=1
            else:
                pic = cv2.cvtColor(cv2.imread(os.path.join(path,img)),cv2.COLOR_BGR2RGB)
                pic = cv2.imread(os.path.join(path,img))
                pic = cv2.resize(pic,size)
                x_test.append([pic])
                y_test[0].append(path2img.index(path))
                counter+=1

    temp0=list(zip(x_train,y_train[0]))
    random.shuffle(temp0)
    x_train,y_train[0]=zip(*temp0)
    temp1=list(zip(x_test,y_test[0]))
    random.shuffle(temp1)
    x_test,y_test[0]=zip(*temp1)

    x_train=np.array(x_train)
    y_train=np.array(y_train)
    x_test=np.array(x_test)
    y_test=np.array(y_test)


    x_train=x_train.reshape(x_train.shape[0],-1).T
    x_test=x_test.reshape(x_test.shape[0],-1).T
    x_train=x_train/255.
    x_test=x_test/255.

    return x_train,x_test,y_train,y_test



from setuptools import setup,find_packages

setup(
    name="projecttalos", # 
    version="0.0.3",
    author="Şeyhmus Baskın",
    author_email="seyhmusbaskin@yandex.com",
    description="A small Machine Learning package",
    url="https://github.com/Xessen/ProjectTalos",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    python_requires=">=3.3",
    install_requires=["numpy>=1.15.0"]
    )


from .activation_functions import *
import numpy as np


def param_init(layer_dims):
    parameters = {}
    L = len(layer_dims)          
    for l in range(1, L):
        parameters['W' + str(l)] = np.random.randn(layer_dims[l], layer_dims[l-1]) / np.sqrt(layer_dims[l-1])
        parameters['b' + str(l)] = np.zeros((layer_dims[l], 1))
          
    return parameters


def forward_activation(A_,W,b,activation):
    Z=np.dot(W,A_)+b
    linear_cache=(A_,W,b)
    A,activation_cache=eval(f"{activation}(Z)")
    cache=(linear_cache,activation_cache)
    return A,cache

def forward_model(X,parameters,layer_func):
    caches = []
    counter=1
    A = X
    numberofcalc=len(layer_func)   
    for i in range(numberofcalc):
        layerfunc,layerrepeat=layer_func[i].split("*")[0],layer_func[i].split("*")[1]
        for y in range(int(layerrepeat)):
            if i==numberofcalc-1 and y==int(layerrepeat)-1:
                AL,cache=forward_activation(A,parameters["W"+str(counter)],parameters["b"+str(counter)],layerfunc)
                caches.append(cache)
                counter+=1
            else:
                A_prev=A
                A,cache=forward_activation(A_prev,parameters["W"+str(counter)],parameters["b"+str(counter)],layerfunc)
                caches.append(cache)
                counter+=1
    return AL,caches

